/*! @file  Packet.h SOMA packet handler

   The Soma Project

   Copyright 2007 Rachel Pasetes, John Burr, Sean Eves Knudsen,
                  Thomas Gerdes, Andrew Parra, Erik Pasternak

   Code adapted from Nanomouse

   (c) 2006 Andrey Yurovsky
            andrey@thedotcommune.com

  This file is part of The Soma Project.

  The Soma Project is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Soma Project is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with The Soma Project; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef PACKET_H
#define PACKET_H

#include "RF.h"

/*  Packet Type defines  */

#define MAP     0x1
#define REQ_MAP 0x2
#define HBEAT   0x3
#define TOFINIT 0x4
#define BEGINIT 0x5
#define PKTACK  0x6
#define TOKPASS 0x7
#define START   0x8
#define NET_IS  0xA
#define DROPPED 0xB
#define REQNET  0xC
#define JUSTADDED 0xD

#define DEBUGPKT 0xFA
#define BUG2     0x33

/* Structures */
typedef struct
{
  uint8_t length;
  uint8_t type;
  uint8_t RoboID;
  uint8_t PacketID;
  uint8_t Data[250];
  uint16_t CheckSum;
} packet;

extern uint8_t isLeader;
extern uint8_t networkElements;

uint8_t bufferAvailable( uint8_t sizeOfPacket );                //verified 100% 
uint16_t generateDataSum(uint8_t* data, uint8_t length);
uint16_t generateSum(packet receivedPacket);                    //verified 100%
uint8_t  checkSum(packet receivedPacket);                        //verified 100%

void enqueuePacket(packet tempPacket);                          //verified 100%  
packet dequeuePacket( void );                                   //verified 100% 

void sendPacket(uint8_t length, uint8_t type, uint8_t* data);   //verified 100%
void incPacketID ( void );
uint8_t hasPackets( void );                                     //verified 100%

void gatherPacket( void );                                      //untested

void packetHandle (void);
void initNetwork (void);
void initLeader (void);
void leaderConfirmNetwork (void);
void leadersNetwork(packet unhandled);

void requestNet( void );

int Packet_Init( void );

void requestNet( void );
void handleDropped( packet unhandled );

void addNewBotToNet(packet unhandled);

void addToNet(packet unhandled);
void rmFromNet( uint8_t RoboID );
void changeLeaderRequestHandle(packet unhandled);
void changeLeader(void);
void changeLeaderAck(packet unhandled);
void checkChangeLeader( void );

void heartBeatHandle(void);
void heartBeatClear(void);
void sendHB(void);
uint8_t getHB(void);

void joinNW(void);

void startUp( void );
void checkNW(void);

void RunNWStateMachine(void);

#endif //PACKET_H
