/*
   The Soma Project

   Copyright 2007 Rachel Pasetes, John Burr, Sean Eves Knudsen,
                  Thomas Gerdes, Andrew Parra, Erik Pasternak

  This file is part of The Soma Project.

  The Soma Project is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Soma Project is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with The Soma Project; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*!@file
   @ingroup command
   @brief SRAM related functions
*/

#include <stdio.h>
#include <avr/io.h>
#include "util.h"

inline void sram_Init( void )
{
   SET_BIT( XMCRA, SRE );   // External SRAM enable
   CLR_BIT( XMCRA, SRW11 ); // setting wait cycles for
   CLR_BIT( XMCRA, SRW10 ); //    SRAM upper sector
   CLR_BIT( XMCRB, XMBK );  // disable bus keeper

   return;
}

#ifdef DEBUG_SRAM

#include "usart.h"
#define MEMSTART ((unsigned char*)0x2200)
#define MEMEND   ((unsigned char*)0xFFFF)

int main(void) {
   volatile unsigned char *memptr;
   volatile unsigned char pattern;
   pattern=0;
   USART0_Init(BAUD57600);
   sram_Init( ); 
   asm("sei");
   printf("Testing memory\n\r");
   printf("Writing pattern:");
   for(memptr=MEMSTART;memptr<MEMEND;memptr++) {
      *memptr=pattern++;
   }
   printf("Checking pattern...\n\r");
   pattern=0;
   for(memptr=MEMSTART;memptr<MEMEND;memptr++) {
      if((unsigned char)(*memptr)!=(unsigned char)(pattern)) {
	     printf("Bad location:%p, %x %x\r\n", memptr, pattern, *memptr );
	  }
	  pattern++;
   }
   printf("Writing Reverse pattern:");
   pattern=0;
   for(memptr=MEMSTART;memptr<MEMEND;memptr++) {
      *memptr=~(pattern);
	  pattern++;
   }
   printf("Checking reverse pattern\r\n");
   pattern=0;
   for(memptr=MEMSTART;memptr<MEMEND;memptr++) {
      if((unsigned char)(*memptr)!=(unsigned char)(~(pattern))) {
	     printf("Bad location:%p, %x %x\r\n", memptr, ~(pattern), *memptr);
	  }
	  pattern++;
   }
   printf("Memory Checking done.\n\r");
   for(;;);
   return 0;
}
#endif //DEBUG_SRAM
