/*
   The Soma Project

   Copyright 2007 Rachel Pasetes, John Burr, Sean Eves Knudsen,
                  Thomas Gerdes, Andrew Parra, Erik Pasternak

  This file is part of The Soma Project.

  The Soma Project is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Soma Project is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with The Soma Project; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*!@file
   @ingroup command
   @brief IC related functions
*/

#define F_CPU_MHZ 16
#define PRESCALE 256
#define PING_SCALE 147
#define INCHES_PER_CYCLE ((int16_t)(((100/F_CPU_MHZ)*PRESCALE)/PING_SCALE)+1) //Note, the +1 compensates for truncation

typedef struct{
   int16_t distance;
   int16_t angle;
}ping_vector;

/*!@brief Initialize Input Capture 1

   This function initializes the Input Capture 1 to allow for
   interrupt driven input.  It is used for the ping sensors.
   The trigger is set to go off on a toggle high.
*/
inline void ic_Init( void );

/*!@brief Input Capture Return Distance in Inches
   @returns the distance from the sonar ping sensor to an object

   This function occurs whenever the TIMER1 input capture triggers.
   Sets the global variable of ICTimerVal to the time difference
   between rising and falling edge.
*/
int16_t ic_GetDistance( void );

/*!@brief Input Capture Ping Sensor Data Return
   @returns A data structure with distance and angle of an object

   This function converts the data recieved from the ping sensor
   to data relevent towards the center of the robot.  This data
   includes the distance from the center of the robot to an
   onject, and the center of the robot and its angle relative to
   an object.
*/
ping_vector ic_PingSensorData( void );
