/*
   The Soma Project

   Copyright 2007 Rachel Pasetes, John Burr, Sean Eves Knudsen,
                  Thomas Gerdes, Andrew Parra, Erik Pasternak

  This file is part of The Soma Project.

  The Soma Project is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Soma Project is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with The Soma Project; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*!@file
   @ingroup command
   @brief OC related functions
*/


#include "util.h"
#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "usart.h"
#include "oc.h"

int16_t current_servo_angle;

inline void oc_Init( void ){
   SET_BIT( DDRB, DDB6 ); /* Set OCB as output */
   SET_BIT( TCCR1B, WGM13 );
   CLR_BIT( TCCR1B ,WGM12 );
   CLR_BIT( TCCR1A, WGM11 );
   CLR_BIT( TCCR1A ,WGM10 );
   //WGMn3:0 = 8
   SET_BIT( TCCR1A, COM1B1 );
   CLR_BIT( TCCR1A, COM1B0 );
   CLR_BIT( TCCR1B, CS12 ); /* Clock select, 8 prescale */
   SET_BIT( TCCR1B, CS11 );
   CLR_BIT( TCCR1B, CS10 );
   ICR1 = 20000;
   
}

void oc_TurnServo( int16_t degree ){
   current_servo_angle = degree;
	OCR1B = 500 + ((int32_t)983 * (int32_t)degree)/100;
}

int16_t oc_GetServoAngle( void ){
   return current_servo_angle;
}

#ifdef DEBUG_OC
int main(void) {
	int i,j;
   int8_t value;
   USART0_Init(BAUD57600);
   asm("sei");
   printf("Hello World\n\r");
   oc_Init();
/*	while (1) {
		OCR1B = 500;
		for( i = 0; i<200; i++ ){
	      	_delay_ms(10);
		}
		OCR1B = 2270;
		for( i = 0; i<200; i++ ){
	      	_delay_ms(10);
		}
	}*/
   while(1){
      getchar();
      oc_TurnServo(0);
      printf("0 degrees\n\r");
      getchar();
      oc_TurnServo(90);
      printf("90 degrees\n\r");
      getchar();
      oc_TurnServo(180);
      printf("180 degrees\n\r");
   }

}
#endif
