/*
   The Soma Project

   Copyright 2007 Rachel Pasetes, John Burr, Sean Eves Knudsen,
                  Thomas Gerdes, Andrew Parra, Erik Pasternak

  This file is part of The Soma Project.

  The Soma Project is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Soma Project is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with The Soma Project; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*!@file
   @brief drivers for generating a 24k and 36.7k signal.

   @sa localize
   @ingroup localize
*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include "SignalGeneration.h"

#define PIN24K PB4
#define PIN36K PB5
#define OCPORT    PORTB
#define WGMA0      0
#define WGM2      3
#define WGM3      4
#define COMA1     7
#define COMA0     6
#define WGM1    1
#define TOIE    0  
#define MAXPULSES 15


/*!@function
   @brief Sets up Timer 1 and Timer 2 for generating a 24k and 36.7k signal.

   Make sure interrupts are disabled when calling this function.
*/
void SG_Init(void) {
   //set up timer2 for PWM, phase correct mode with no prescaling.
   
   TCCR2B = (1<<WGM22) | (1<<CS20);
   TCCR2A = (1<<WGM20);// | (1<<WGM21);
   //Set the top of Timer2 to 218 to create a 36.7k signal
   OCR2A = 167;
  //set up timer1 for PWM, phase correct mode with no prescaling.
   TCCR1B = (1<<WGM13) | (1<<CS10);
   TCCR1A = (1<<WGM10) | (1<<WGM11);
   //set the top of Timer1 to 333 to generate a 24k signal
   OCR1A = 109;
   //enable the output for both pins
   DDRB |= (1<<DDB4) | (1<<DDB5);
}

void SG_Start(void) {
   //reset the two timers
   TCNT1 = 0;
   TCNT2 = 0;
   //Enable the timer for both pins
   TCCR2A |= (1<<COM2A0);
   TCCR1A |= (1<<COM1A0);

   //enable the interrupt for Timer1, we won't bother handling anything
   //on Timer2s interrupts. This interrupts when bottom is reached.
   TIMSK2 = (1<<TOIE);
//   TIMSK2 = (1<<TOIE);
}


ISR(TIMER2_OVF_vect) { 
   //keep track of the number of times this interrupt has occured.
   static int pulses = 0;
   if(pulses++ == MAXPULSES) {
      //if it's finished emitting turn off the signals.
      //DDRB &= ~((1<<DDB4) | (1<<DDB5));
      TCCR1A &= ~(1<<COM1A0);
      TCCR2A &= ~(1<<COM2A0);
      //turn off the interrupt
      TIMSK2 &= ~(1<<TOIE);
//      TIMSK2 &= ~(1<<TOIE);
      pulses = 0;

   }
}

