/*
   The Soma Project

   Copyright 2007 Rachel Pasetes, John Burr, Sean Eves Knudsen,
                  Thomas Gerdes, Andrew Parra, Erik Pasternak

  This file is part of The Soma Project.

  The Soma Project is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Soma Project is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with The Soma Project; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*!@defgroup localize Localization module
   @file
   @ingroup localize
   @brief Main program for Soma robots

  This file contains run code for the Localization Module.
*/


//#define ATMEGA1281   //Sets up the defines for a 16MHz Atmel Clock

/* Includes */


#include "usart.h"
#include <avr/io.h>
#include <stdio.h>
#include <util/delay.h>
#include "irobot.h"
#include "spi.h"
#include "../share/util.h"
#include "SignalGeneration.h"
#include "localcom.h"
#include "triglookup.h"
#include "TOFcom.h"

typedef struct{
   int32_t x;
   int32_t y;
   int16_t angle;
} xy_coord;

unsigned char Data[8];

xy_coord getPosition(xy_coord position){
   int16_t angle = iRobot_GetAngle();
   int32_t distance = iRobot_GetDistance();  //Converts the distance to mm
   
   position.angle = (360+(position.angle+angle)%360)%360;
   position.x += ((distance)*((int32_t)cosd(position.angle)));
   position.y += ((distance)*((int32_t)sind(position.angle)));
 
   return position;
}

xy_coord RoboPos;

void initAll(void) {
   asm("cli");
   USART1_Init( BAUD57600 ); // Com to iRobot
   SPI_Init(SLAVE);
   iRobot_Init();
   iRobot_ModeFull();
   asm("sei");
   RoboPos.x = (int32_t)(((int32_t)-200) << 15);
   //RoboPos.x = 0;
   RoboPos.y = 0;
   RoboPos.angle = 90;
}

void handleSPI( void ) {
   uint8_t cmd;
   int32_t x, y;
   cmd = SPI_Transfer(NO_DATA);
   switch(cmd) {
      case SEND_COMMAND:
         while (cmd == SEND_COMMAND) cmd = SPI_Transfer( SPI_ACK );
         if( cmd == 'w' )
            iRobot_Drive(200, 200);
         else if( cmd == 'a' )
            iRobot_Drive(-200, 200);
         else if( cmd == 's' )
            iRobot_Drive(-200, -200);
         else if( cmd == 'd' )
            iRobot_Drive(200, -200);
         else
            iRobot_Drive(0, 0);
         break;
      case GETXY:
         x = ( RoboPos.x/10>>15);
         y = ( RoboPos.y/10>>15);
         while(cmd == GETXY) cmd = SPI_Transfer(SPI_ACK);
         SPI_Transfer(0);
         SPI_Transfer(x>>8);
         SPI_Transfer(x);
         SPI_Transfer(y>>8);
         SPI_Transfer(y);
         SPI_Transfer(RoboPos.angle>>8);
         SPI_Transfer(RoboPos.angle);
         break;
   }

}

int main(int argc, char**argv){
   initAll();
   for(;;){
      if(SPI_TransferComplete()) handleSPI();
      RoboPos = getPosition(RoboPos);
      _delay_ms(10);
   }
}

#ifdef NAVIGATE //{{{
int main( void ) {
   int i, j;
   asm("cli");
   USART0_Init( BAUD57600 );
   USART1_Init( BAUD57600 );
   SPI_Init(SLAVE);
   iRobot_Init();
   iRobot_ModeFull();
   asm("sei");
   _delay_sec(1);
   for(;;){
      if(SPI_Transfer(NODATA) == SEND_COMMAND){
         iRobot_Drive(-200,200);
         iRobot_GetAngle();   //Clears GetAngle
         for(i = 0; i<90; i += iRobot_GetAngle()){
            _delay_ms(16);
         }
         SPI_Transfer(SPI_ACK);
         SPI_Transfer(SPI_ACK);
      }
      else
         iRobot_Drive(200,200);
   }
}
#endif //}}}
/*
#ifdef NAVIGATE_AND_MAP 
int main( void ) {
   int i, j;
   asm("cli");
   USART0_Init( BAUD57600 );
   USART1_Init( BAUD57600 );
   SPI_Init(SLAVE);
   iRobot_Init();
   iRobot_ModeFull();
   asm("sei");
   _delay_sec(1);
   for(;;){/*
      
      
      if(SPI_Transfer(NO_DATA) == ROBOTTURN90){
         iRobot_Drive(-200,200);
         iRobot_GetAngle();   //Clears GetAngle
         for(i = 0; i<90; i += iRobot_GetAngle()){
            _delay_ms(16);

         }
         SPI_Transfer(SPI_ACK);
         SPI_Transfer(SPI_ACK);
      }
      else if(SPI_Transfer(NO_DATA) == ROBOTSTOP){
         iRobot_Drive(0,0);
      }
      else if(SPI_Transfer(NO_DATA) == ROBOTDRIVE){
         iRobot_Drive(200,200);
      }
   }
}
#endif */
