#include <Windows.h>
#include <stdio.h>
#include "usart.h"
#include "../command/Packet.h"

uint8_t Com_RxBuf[128];
uint8_t Com_TxBuf[128];
uint8_t Com_RxHead, Com_RxTail;
uint8_t Com_TxHead, Com_TxTail;

HANDLE comPort;
HANDLE terminateEvent;

DWORD WINAPI ReadThread(LPVOID lpParam) {
   OVERLAPPED osReader = {};
   HANDLE hArray[2];
   BOOL fThreadDone = FALSE;
   BOOL fWaitingOnRead = FALSE;
   long dwRes;
   char data;
   long len;

   osReader.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
   hArray[0] = osReader.hEvent;
   hArray[1] = terminateEvent;

   while (!fThreadDone) {
      if(!fWaitingOnRead) {
         if(!ReadFile(comPort, &data, 1, &len, &osReader)) {
            fWaitingOnRead = TRUE;
         } else {
            Com_RxBuf[(Com_RxHead+1)&127] = data;
            Com_RxHead = (Com_RxHead+1)&127;
         }
      }
      if(fWaitingOnRead) {
         dwRes = WaitForMultipleObjects(2, hArray, FALSE, INFINITE);
         if(dwRes == WAIT_OBJECT_0) {
            if(GetOverlappedResult(comPort, &osReader, &len, FALSE)) {
               Com_RxBuf[(Com_RxHead+1)&127] = data;
               Com_RxHead = (Com_RxHead+1)&127;
            }
            fWaitingOnRead=FALSE;
         } else if (dwRes == WAIT_OBJECT_0 + 1) {
            ResetEvent(terminateEvent);
            fThreadDone = TRUE;
         }
      }
   }
   
   return;
}

uint8_t USART0_Receive( void ) {
   while(Com_RxHead == Com_RxTail) Sleep(1);
   Com_RxTail = (Com_RxTail + 1) & (128-1);
   return Com_RxBuf[Com_RxTail];
}

void USART0_Transmit( uint8_t data ) {
   OVERLAPPED osWrite = {0};
   HANDLE hArray[2];
   DWORD dwWritten;
   DWORD dwRes;
   long len;
   osWrite.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
   hArray[0] = osWrite.hEvent;
   hArray[1] = terminateEvent;
   if(!WriteFile(comPort, &data, 1, &len, &osWrite)) {
      dwRes = WaitForMultipleObjects(2, hArray, FALSE, INFINITE);
      switch(dwRes) {
         case WAIT_OBJECT_0:
            GetOverlappedResult(comPort, &osWrite, &dwWritten, FALSE);
            break;
         case WAIT_OBJECT_0 + 1:
            break;  //if terminate, just quit.
      }
   }
   CloseHandle(osWrite.hEvent);
}

void USART0_SetBaud(enum BaudRate br)  {
   return;
}

uint8_t USART0_SendingData(void) {
   return 0;
}

uint8_t USART0_HasData( void ) {
   return (Com_RxHead != Com_RxTail);
}

void USART0_Flush ( void ) {
   Com_TxHead = Com_TxTail;
}

inline void CMD_ON() {
   EscapeCommFunction(comPort, SETRTS);
}
inline void CMD_OFF() {
   EscapeCommFunction(comPort, CLRRTS);
}

int USART0_Init(int port) {
   long idRead;
   char portnum[5] = "COM1";
   portnum[3] = '0' + port;
   comPort = CreateFile(portnum, GENERIC_READ | GENERIC_WRITE, 0, NULL,
                        OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL);
   COMMTIMEOUTS commTimeout;
   DCB config;
   GetCommState(comPort, &config);
   config.BaudRate    = CBR_57600;
   config.StopBits    = ONESTOPBIT;
   config.Parity      = NOPARITY;
   config.ByteSize    = 8;
   config.fBinary     = TRUE;
   config.fParity     = TRUE;
   if (!SetCommState(comPort, &config))
      return 1;
   if(GetCommTimeouts(comPort, &commTimeout)) {
      commTimeout.ReadIntervalTimeout = 1000;
   }
   SetCommTimeouts(comPort, &commTimeout);
   EscapeCommFunction(comPort, CLRRTS);
   Com_RxHead = Com_RxTail = Com_TxHead = Com_TxTail = 0;
   terminateEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
   CreateThread(NULL, 0, ReadThread, NULL, 0, &idRead);
   return 0;
}
/*
int main(int argc, char **argv) {
   char c;
   int counter;
   int i;
   packet pkt;
   uint8_t text0[19]="Hello World!";

   USART0_Init(BAUD57600);
   Packet_Init();

   sendPacket(0x13, 0x2, text0);

   while(1) {
      if(hasPackets()){
         pkt = dequeuePacket();
         printf("Got Packet\n");
         printf("%d '%s'\n", pkt.RoboID,  pkt.Data);
      }
      Sleep(1);
   };
   return 0;
}

*/
