#include <avr/io.h>
#include "spi.h"

#define DDR_SPI DDRB
#define DD_SS   DDB0
#define DD_MOSI DDB2
#define DD_MISO DDB3
#define DD_SCK  DDB1


inline void SPI_Init(char mode) {
   if(mode==MASTER) {
      DDR_SPI = _BV(DD_MOSI) | _BV(DD_SCK) | _BV(DD_SS);
      PORTB |= _BV(PB0);
      /* Enable SPI, master, and set clock to fsck / 2 */
      SPCR=_BV(SPE) | _BV(MSTR); //| _BV(SPR0) ; /* fsck / 4 */
      //SPSR = 1; /* fsck / 2 */
   } else {
      DDR_SPI = _BV(DD_MISO);
      SPCR  =_BV(SPE); //We won't be using interrupt driven SPI
      SPDR = 0;
   }
}


unsigned char SPI_TransferComplete( void )
{
   return SPSR & (_BV(SPIF));
}

uint8_t SPI_ReadData( void )
{
   return SPDR;
}


unsigned char SPI_Transfer(unsigned char data)
{
   /*do {
      SPDR = data;
   } while ( SPSR & (1<<WCOL) );*/

   SPDR = data;
   while( (SPSR & (1<<WCOL)) ) {
      SPDR = data;
   }

   while(!(SPSR & (1<<SPIF)));
   return SPDR;
}


void SPI_TransferBlock(unsigned char* Dout, unsigned char *Din)
{
   int i;
   for(i = 0; i < 8; i++) {
      Din[i] = SPI_Transfer(Dout[i]);
   }
}

