/*
   The Soma Project

   Copyright 2007 Rachel Pasetes, John Burr, Sean Eves Knudsen,
                  Thomas Gerdes, Andrew Parra, Erik Pasternak

  This file is part of The Soma Project.

  The Soma Project is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Soma Project is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with The Soma Project; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*!@file
   @ingroup command
   @brief SPI interface module
*/

#ifndef _SPI_H_
#define _SPI_H_

#define MASTER 0
#define SLAVE 1

/*!@fn inline void SPI_Init(char mode)
   @brief Initializes the SPI.
   @This function sets the transfer clock to fosc/4
   @and enables the SPI as either master or slave.
   @Interrupts are not enabled for the SPI.
*/
inline void SPI_Init(char mode);

/*!@fn void SPI_TransferBlock(unsigned char* Dout, unsigned char *Din)
   @brief Transfers 8 bytes on the SPI.
   @Takes in a pointer to an output buffer and input buffer.
   @8 bytes are transferred out of the output buffer and
   @8 bytes are read into the input buffer.
*/
void SPI_TransferBlock(unsigned char* Dout, unsigned char *Din);

/*!@fn unsigned char SPI_Receive(void)
   @brief Reads the SPI Data register.
*/
//unsigned char SPI_Receive(void);

/*!@fn void SPI_Transmit(unsigned char data)
   @brief writes a byte to the SPI.
   @Writes a byte to the SPI and then waits for transfer to complete.
*/
unsigned char SPI_Transfer(unsigned char data);

/*!@fn unsigned char SPI_TransferComplete( void )
   @brief Returns the status of the SPIF
*/
unsigned char SPI_TransferComplete( void );

/*!@fn uint8_t SPI_ReadData( void )
   @brief Returns SPDR
*/
uint8_t SPI_ReadData( void );

#endif
