/*
   The Soma Project

   Copyright 2007 Rachel Pasetes, John Burr, Sean Eves Knudsen,
                  Thomas Gerdes, Andrew Parra, Erik Pasternak

  This file is part of The Soma Project.

  The Soma Project is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Soma Project is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with The Soma Project; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*!@file
   @ingroup localize
   @brief Trig Lookup Table

   This file contains functions relating to trig.
*/

#include "triglookup.h"

int16_t sin_Lookup [] = {0, 572, 1144, 1715, 2286, 2856, 3425, 3993, 4560, 
5126, 5690, 6252, 6813, 7371, 7927, 8481, 9032, 9580, 10126, 10668, 11207, 
11743, 12275, 12803, 13328, 13848, 14365, 14876, 15384, 15886, 16384, 
16877, 17364, 17847, 18324, 18795, 19261, 19720, 20174, 20622, 21063, 
21498, 21926, 22348, 22763, 23170, 23571, 23965, 24351, 24730, 25102, 
25466, 25822, 26170, 26510, 26842, 27166, 27482, 27789, 28088, 28378, 
28660, 28932, 29197, 29452, 29698, 29935, 30163, 30382, 30592, 30792, 
30983, 31164, 31336, 31499, 31651, 31795, 31928, 32052, 32166, 32270, 
32365, 32449, 32524, 32588, 32643, 32688, 32723, 32748, 32763, 32767};

int16_t sind(int16_t degree){
   degree = (degree%360 +360)%360;
   if(degree<90){
      return(sin_Lookup[degree]);
   }
   else if(degree<180){
      degree = 180-degree;
      return(sin_Lookup[degree]);
   }
   else if(degree<270){
      degree = degree-180;
      return(-sin_Lookup[degree]);
   }
   else{
      degree = 360-degree;
      return(-sin_Lookup[degree]);
   }
}

int16_t cosd(int16_t degree){
   return(sind(degree+90));
}

int16_t sinInversed(int16_t value){
   int i;
   int j = 0;
   int16_t degree = 0;
   if(value<0){
      value = ~value;
      j = 1;
   }
   for(i = 0; i<90; i++){
      if(value < sin_Lookup[i]){
         degree = i-1;
         break;
      }
   }
   if(i == 90)
      degree = 90;
   if(j)
      return -degree;
   else
      return degree;
}
