/*
   The Soma Project

   Copyright 2007 Rachel Pasetes, John Burr, Sean Eves Knudsen,
                  Thomas Gerdes, Andrew Parra, Erik Pasternak

  This file is part of The Soma Project.

  The Soma Project is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Soma Project is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with The Soma Project; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*!@file
   @ingroup command
   @brief Trig Lookup code, sin and cos functions
*/

#ifndef TRIGLOOKUP_H
#define TRIGLOOKUP_H

#include <stdint.h>

/*!@fn int16_t sind(int16_t degree)
   @brief Returns sin of a degree
   @This function uses a lookup table to return a
   @sin of a degree.  It returns a 16 bit value
   @that needs to be shifted by 10 bits to the
   @right.
*/
int16_t sind(int16_t degree);

/*!@fn int16_t cosd(int16_t degree)
   @brief Returns sin of a degree
   @This function uses a lookup table to return a
   @cos of a degree.  It returns a 16 bit value
   @that needs to be shifted by 10 bits to the
   @right.
*/
int16_t cosd(int16_t degree);

/*!@fn sinInversed(int16_t value)
   @brief Returns arcsin of a value
   @This function uses a lookup table to return a
   @arcsin of a degree.  It expects the value to
   @be initialy multiplied by 1024.
*/
int16_t sinInversed(int16_t value);

#endif
