/*
   The Soma Project

   Copyright 2007 Rachel Pasetes, John Burr, Sean Eves Knudsen,
                  Thomas Gerdes, Andrew Parra, Erik Pasternak

  This file is part of The Soma Project.

  The Soma Project is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Soma Project is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with The Soma Project; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*!@file
   @brief Utility functions header
   @ingroup lib
*/
#ifndef _UTIL_H
#define _UTIL_H

/* bitmasking */
#define SET_BIT( reg, bit ) reg |= (1<<bit)
#define CLR_BIT( reg, bit ) reg &= ~(1<<bit)
#define BIT_IS_SET( reg, bit )  ( reg & (1<<bit) )
#define BIT_IS_CLR( reg, bit )  !( reg & (1<<bit) )
#define TOGGLE_BIT( reg, bit ) reg ^= (1<<bit)

/*!@fn void charToHex(char *buf, char val)
   @param buf Character buffer to output to
   @param val Value to output
   @brief Print one byte as two hex characters in buf.

   When this function is called, buf[0:1] are set with the value in val.
*/
void charToHex(char *buf, char val);
/*!@fn void shortToHex(char *buf, short val)
   @param buf Character buffer to output to
   @param val Value to output
   @brief Print two bytes as four hex characters in buf.

   When this function is called, buf[0:3] are set with the value in val.
*/
void shortToHex(char *buf, short val);
/*!@fn void intToHex(char *buf, int val)
   @param buf Character buffer to output to
   @param val Value to output
   @brief Print four bytes as eight hex characters in buf.

   When this function is called, buf[0:7] are set with the value in val.
*/
void intToHex(char *buf, int val);
/*!@fn void _delay_sec( int time )
   @param time Time in seconds to delay
   @brief Creates a one second delay.

*/
void _delay_sec( int time );


#endif
