/*
   The Soma Project

   Copyright 2007 Rachel Pasetes, John Burr, Sean Eves Knudsen,
                  Thomas Gerdes, Andrew Parra, Erik Pasternak

  This file is part of The Soma Project.

  The Soma Project is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Soma Project is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with The Soma Project; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*!@file main.c
   @ingroup tof
   @brief State machine that controls the US module.

   Waits for a message from the Localize module. If it's the signal that IR
   is expected, it gathers the IC data, then processes it and waits for the next
   command.
   If it's the return data message it returns the data it recorded or NODATA.
*/

#include <avr/io.h>
#include <util/delay.h>
//#include <avr/interrupt.h>
//#include <stdio.h>
#include "../share/util.h"
#include "../share/TOFcom.h"
#include "usi.h"
#include "tcnt1.h"
#include "tof.h"

int main ( void )
{
   uint8_t cmd, reply, result;
   uint8_t temp = 0;
   uint32_t lowCount;
   uint16_t IROnTime, USTimeOfFlight;

   asm("cli");
   USI_InitSPISlave( SPI_MODE );
   // init to toggle edge type trigger after each event
   TCNT1_InitIC( IC_EDGE_TYPE, 1 );
   TCNT1_DisableIC();
   asm("sei");

   uint8_t i = 0;

   while ( 1 ) {
      USI_WaitForBufferEmpty();
      USI_Transmit( NO_DATA );
      USI_WaitForBufferEmpty();
      cmd = USI_GetReceivedData();
      switch ( cmd ) {
         case EXPECTIR:
            USI_Transmit( EXPECTIR_ACK );
            USI_Transmit( NO_DATA );

            lowCount = 0;
            // wait for long low time
            while ( lowCount < 100000 ) {
               if ( BIT_IS_CLR( PINA, PINA7 ) ) {
                  lowCount++;
               } else {
                  lowCount = 0;
               }
               _delay_us( 1 );
            }

            TCNT1_SetICOnRise();
            TCNT1_EnableIC();

            result = TOF_GatherTOFReadings();
            //result = 0; 

            TCNT1_DisableIC();

            IROnTime = TOF_Readings.IRFinish - TOF_Readings.IRStart;
            USTimeOfFlight = TOF_Readings.USArrival - TOF_Readings.IRStart;
            /*IROnTime = 0x1234;
            USTimeOfFlight = 0x5678;*/

            //_delay_sec(1);
            
            USI_WaitForBufferEmpty();
            USI_ClearTransmitComplete();
            if(result) {
               // timeout occurred
               do {
                  USI_Transmit( TOF_INVALID );
                  USI_WaitForTransmitComplete();
                  reply = USI_GetReceivedData();
               } while ( reply != TOF_INVALID_ACK );
               USI_Transmit( 0x0A );
               USI_Transmit( 0x0A );
               USI_Transmit( 0x0A );
               USI_Transmit( 0x0A );
            } else {
               do {
                  USI_Transmit( TOF_DATA );
                  USI_WaitForTransmitComplete();
                  reply = USI_GetReceivedData();
               } while ( reply != TOF_DATA_ACK );
               USI_Transmit( (uint8_t)(IROnTime>>8) );
               USI_Transmit( (uint8_t)(0x00FF & IROnTime) );
               USI_Transmit( (uint8_t)(USTimeOfFlight>>8) );
               USI_Transmit( (uint8_t)(0x00FF & USTimeOfFlight) );
            }
            break;
         default:
            ;
      }
   }
   return 0;
}
