/*
   The Soma Project

   Copyright 2007 Rachel Pasetes, John Burr, Sean Eves Knudsen,
                  Thomas Gerdes, Andrew Parra, Erik Pasternak

  This file is part of The Soma Project.

  The Soma Project is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Soma Project is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with The Soma Project; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*!@file tcnt1.h
   @brief Timer/Counter0 header
   @ingroup tof
*/

#ifndef _TCNT1_H
#define _TCNT1_H

#include <avr/io.h>
#include "../share/util.h"


/*!@brief Disables the input capture interrupt.
 */
#define TCNT1_DisableIC() CLR_BIT( TIMSK1, ICIE1 )

/*!@brief Sets input capture to occur on rising edge.
 */
#define TCNT1_SetICOnRise() SET_BIT( TCCR1B, ICES1 )

/*!@brief Sets input capture to occur on falling edge.
 */
#define TCNT1_SetICOnFall() CLR_BIT( TCCR1B, ICES1 )

void TCNT1_InitIC( uint8_t edgeType, uint8_t edgeTypeToggle );
uint16_t TCNT1_WaitForICOccurred( void );
inline void TCNT1_EnableIC( void );

#endif   //ifndef _TCNT1_H
