/*
   The Soma Project

   Copyright 2007 Rachel Pasetes, John Burr, Sean Eves Knudsen,
                  Thomas Gerdes, Andrew Parra, Erik Pasternak

  This file is part of The Soma Project.

  The Soma Project is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Soma Project is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with The Soma Project; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*!@defgroup tof Time of Flight module

   @file tof.c
   @ingroup tof
*/


#include <avr/io.h>
#include <util/delay.h>
//#include <avr/interrupt.h>
//#include <stdio.h>
#include "../share/util.h"
#include "../share/TOFcom.h"
#include "usi.h"
#include "tcnt1.h"
#include "tof.h"

/*!@brief Blocking function that waits for the sequential 
          rising, falling, rising edges expected from the IR and US.
    
    Stores event times to TOF_Readings struct.
    Will timeout if the wait for any of the three signals is longer
    than 32ms.

   @returns 1 if any reading timed out, 0 if successful
    ____              _   _
   |    |            | | | |
   |    |            | | | |
___|    |____________| |_| |_
    \    \ IRFinish   \ USArrival
     IR Start                         */
uint8_t TOF_GatherTOFReadings( void )
{
   TOF_Readings.IRStart = TCNT1_WaitForICOccurred();
   TOF_Readings.IRFinish = TCNT1_WaitForICOccurred();
   TOF_Readings.USArrival = TCNT1_WaitForICOccurred();
   /*if ( (TOF_Readings.IRStart==0) || 
        (TOF_Readings.IRFinish==0) || 
        (TOF_Readings.USArrival==0) ) {
      return 1;
   } else {
      return 0;
   }*/
   return 0;
}

#ifdef MASTER_TEST
#warning compiling for MASTER_TEST
int main ( void ) 
{
   asm("cli");

   SET_BIT( DDRA, PA0 );
   SET_BIT( PORTA, PA0 );
   SET_BIT( DDRB, PB2 );
   SET_BIT( PORTB, PB2 );
   _delay_ms( 100 );

   USI_InitSPIMaster( SPI_MODE );
   asm("sei");

   uint8_t i = 0;

   while ( 1 ) {
      //_delay_ms( 10 );

      /*CLR_BIT( PORTA, PA0 );
      USI_Transmit( i++ );
      USI_Transmit( i++ );
      USI_WaitForBufferEmpty();
      SET_BIT( PORTA, PA0 );*/

      _delay_us( 500 ); 

      CLR_BIT( PORTB, PB2 );
      USI_Transmit( i++ );
      USI_Transmit( i++ );
      USI_WaitForBufferEmpty();
      _delay_us( 20 );
      SET_BIT( PORTB, PB2 );
   }

   return 0;
}
#elif SLAVE_TEST
#warning compiling for SLAVE_TEST
int main ( void ) 
{
   asm("cli");

   USI_InitSPISlave( SPI_MODE );
   asm("sei");

   uint8_t i = 0;

   /*while ( 1 ) {
      PORTA ^= (1<<PA2);
   }*/

   while ( 1 ) {
      USI_Transmit( i++ );
      USI_Transmit( i++ );
      USI_Transmit( i++ );
      USI_Transmit( i++ );
      USI_Transmit( i++ );
      USI_Transmit( i++ );
      USI_Transmit( i++ );
      USI_Transmit( i++ );
      USI_Transmit( i++ );
      USI_Transmit( i++ );
      USI_Transmit( i++ );
      USI_Transmit( i++ );
      USI_WaitForBufferEmpty();
      //_delay_ms( 10 );
   }

   return 0;
}
#elif IC_TEST
#warning compiling for IC_TEST
int main ( void )
{
   uint16_t lowCount;
   uint16_t IROnTime, USTimeOfFlight;
   unsigned char result;

   SET_BIT( DDRA, PA0 );
   SET_BIT( PORTA, PA0 );

   SET_BIT( DDRA, PA1 );
   SET_BIT( PORTA, PA1 );

   asm("cli");
   USI_InitSPIMaster( SPI_MODE );

   // init to toggle edge type trigger after each event
   TCNT1_InitIC( IC_EDGE_TYPE, 1 );
   TCNT1_DisableIC();

   asm("sei");

   while ( 1 ) {
      lowCount = 0;
      // wait for long low time
      while ( lowCount < 10 ) {
         if ( BIT_IS_CLR( PINA, PINA7 ) ) {
            lowCount++;
         } else {
            lowCount = 0;
         }
         _delay_us( 10 );
      }
      TCNT1_SetICOnRise();
      
      TCNT1_EnableIC();
      
      result = TOF_GatherTOFReadings();

      TCNT1_DisableIC();
      if(result) {
         CLR_BIT( PORTA, PA0 );
         USI_Transmit(NO_DATA);
         SET_BIT( PORTA, PA0 );
      }
      else {
         IROnTime = TOF_Readings.IRFinish - TOF_Readings.IRStart;
         USTimeOfFlight = TOF_Readings.USArrival - TOF_Readings.IRStart;
         CLR_BIT( PORTA, PA0 );
         USI_Transmit( (uint8_t)(IROnTime>>8) );
         USI_Transmit( (uint8_t)(0x00FF & IROnTime) );
         USI_Transmit( (uint8_t)(USTimeOfFlight>>8) );
         USI_Transmit( (uint8_t)(0x00FF & USTimeOfFlight) );
         USI_WaitForBufferEmpty();
         SET_BIT( PORTA, PA0 );
      }
   }

   return 0;
}
#elif TOF_TEST
#warning compiling for TOF_TEST
int main ( void )
{
   uint16_t lowCount;
   uint16_t IROnTime, USTimeOfFlight;

   unsigned char result;

   asm("cli");
   USI_InitSPISlave( SPI_MODE );

   // init to toggle edge type trigger after each event
   TCNT1_InitIC( IC_EDGE_TYPE, 1 );
   TCNT1_DisableIC();

   asm("sei");

   while ( 1 ) {
      lowCount = 0;
      // wait for long low time
      while ( lowCount < 10 ) {
         if ( BIT_IS_CLR( PINA, PINA7 ) ) {
            lowCount++;
         } else {
            lowCount = 0;
         }
         _delay_us( 10 );
      }

      
      TCNT1_SetICOnRise();
      
      TCNT1_EnableIC();
      
      result = TOF_GatherTOFReadings();

      TCNT1_DisableIC();
      if(result) {
         USI_Transmit(NO_DATA);  // will receive instruction here
         
         USI_Transmit(NO_DATA);
         USI_Transmit(NO_DATA);
         USI_Transmit(NO_DATA);
         USI_Transmit(NO_DATA);
         USI_Transmit(NO_DATA);
      }
      else {
         IROnTime = TOF_Readings.IRFinish - TOF_Readings.IRStart;
         USTimeOfFlight = TOF_Readings.USArrival - TOF_Readings.IRStart;
         
         USI_Transmit(NO_DATA);  // will receive instruction here
         
         USI_Transmit( TOF_DATA );
         USI_Transmit( (uint8_t)(IROnTime>>8) );
         USI_Transmit( (uint8_t)(0x00FF & IROnTime) );
         USI_Transmit( (uint8_t)(USTimeOfFlight>>8) );
         USI_Transmit( (uint8_t)(0x00FF & USTimeOfFlight) );
      }
      USI_WaitForBufferEmpty();
   }

   return 0;
}
#elif SPITOLOCALTEST
#warning compiling for SPITOLOCALTEST
int main ( void )
{
   uint8_t cmd, reply, result;
   uint8_t temp = 0;
   uint16_t lowCount;
   uint16_t IROnTime, USTimeOfFlight;

   asm("cli");
   USI_InitSPISlave( SPI_MODE );
   // init to toggle edge type trigger after each event
   TCNT1_InitIC( IC_EDGE_TYPE, 1 );
   TCNT1_DisableIC();
   asm("sei");

   uint8_t i = 0;

   while ( 1 ) {
      USI_WaitForBufferEmpty();
      USI_Transmit( NO_DATA );
      USI_WaitForBufferEmpty();
      cmd = USI_GetReceivedData();
      switch ( cmd ) {
         case EXPECTIR:
            USI_Transmit( EXPECTIR_ACK );
            USI_Transmit( NO_DATA );

            /*lowCount = 0;
            // wait for long low time
            while ( lowCount < 10 ) {
               if ( BIT_IS_CLR( PINA, PINA7 ) ) {
                  lowCount++;
               } else {
                  lowCount = 0;
               }
               _delay_us( 10 );
            }*/

            TCNT1_SetICOnRise();
            TCNT1_EnableIC();

            result = TOF_GatherTOFReadings();
            //result = 0; 

            TCNT1_DisableIC();

            IROnTime = TOF_Readings.IRFinish - TOF_Readings.IRStart;
            USTimeOfFlight = TOF_Readings.USArrival - TOF_Readings.IRStart;
            /*IROnTime = 0x1234;
            USTimeOfFlight = 0x5678;*/

            _delay_sec(1);
            
            USI_WaitForBufferEmpty();
            USI_ClearTransmitComplete();
            if(result) {
               // timeout occurred
               do {
                  USI_Transmit( TOF_INVALID );
                  USI_WaitForTransmitComplete();
                  reply = USI_GetReceivedData();
               } while ( reply != TOF_INVALID_ACK );
               USI_Transmit( 0x0A );
               USI_Transmit( 0x0A );
               USI_Transmit( 0x0A );
               USI_Transmit( 0x0A );
            } else {
               do {
                  USI_Transmit( TOF_DATA );
                  USI_WaitForTransmitComplete();
                  reply = USI_GetReceivedData();
               } while ( reply != TOF_DATA_ACK );
               USI_Transmit( (uint8_t)(IROnTime>>8) );
               USI_Transmit( (uint8_t)(0x00FF & IROnTime) );
               USI_Transmit( (uint8_t)(USTimeOfFlight>>8) );
               USI_Transmit( (uint8_t)(0x00FF & USTimeOfFlight) );
            }
            break;
         default:
            ;
      }
   }
   return 0;
}
#else
   //#error must define MASTER_TEST, SLAVE_TEST, IC_TEST, TOF_TEST, or SPITOLOCALTEST to 1
#endif // #MASTER OR SLAVE or IC
