/*
   The Soma Project

   Copyright 2007 Rachel Pasetes, John Burr, Sean Eves Knudsen,
                  Thomas Gerdes, Andrew Parra, Erik Pasternak

  This file is part of The Soma Project.

  The Soma Project is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Soma Project is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with The Soma Project; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*!@file tof.h
   @brief Time of Flight header
   @ingroup tof
*/

#ifndef _TOF_H
#define _TOF_H

#define SPI_MODE 0	// Sample on leading _rising_ edge, setup on trailing _falling_ edge.
//#define SPI_MODE 1	// Sample on leading _falling_ edge, setup on trailing _rising_ edge.

//#define IC_EDGE_TYPE 0 // trigger on falling edge
#define IC_EDGE_TYPE 1 // trigger on rising edge

typedef struct TOF_ReadingsStruct {
   uint16_t IRStart;
   uint16_t IRFinish;
   uint16_t USArrival;
} TOFReadings_t; 

TOFReadings_t TOF_Readings;

/*!@brief Blocking function that waits for the sequential 
          rising, falling, rising edges expected from the IR and US.
    
    Stores event times to TOF_Readings struct.
    Will timeout if the wait for any of the three signals is longer
    than 32ms.

    ____              _   _
   |    |            | | | |
   |    |            | | | |
___|    |____________| |_| |_
    \    \ IRFinish   \ USArrival
     IR Start                         */
unsigned char TOF_GatherTOFReadings( void );

#endif
